@@ -1,6 +1,6 @@
 import numpy as np
 import pandas as pd
-from scipy.stats import binom, norm, poisson, beta
+from scipy.stats import (binom_test, beta, multinomial, poisson)
 
 class HC(object) :
     """
@@ -28,16 +28,19 @@ class HC(object) :
     -------
         HC       HC and P-value attaining it
         HCstar   sample adjustet HC (HCdagger in [1])
+        HCjin    a version of HC from 
+                [2] Jiashun Jin and Wanjie Wang, "Influential features PCA for
+                 high dimensional clustering"
         
-
     """
     def __init__(self, pvals, stbl=True) :
 
         self._N = len(pvals)
-        assert(self._N >0)
+        assert(self._N > 0), "list of pvals cannot be empty!"
 
         EPS = 1 / self._N
         self._EPS = EPS
+        self._istar = 1
 
         self._pvals = np.sort(np.asarray(pvals.copy()))
         self._uu = np.linspace(1 / self._N, 1-EPS, self._N)
@@ -53,12 +56,13 @@ class HC(object) :
     def _calculateHC(self, imin, imax) :
         if imin > imax :
             return np.nan
-        istar = np.argmax(self._zz[imin:imax]) + imin
-        zMaxStar = self._zz[istar]
-        return zMaxStar, self._pvals[istar]
+        self._istar = np.argmax(self._zz[imin:imax]) + imin
+        zMaxStar = self._zz[self._istar]
+        return zMaxStar, self._pvals[self._istar]
 
     def HC(self, gamma=0.2) : 
-        """higher criticism score
+        """
+        Higher Criticism score
 
         Args:
         -----
@@ -74,7 +78,25 @@ class HC(object) :
                         int(np.floor(gamma * self._N + 0.5)))        
         return self._calculateHC(imin, imax)
 
-    def BJ(self, gamma=0.1) :
+    def HCjin(self, gamma=0.2) :
+        """sample-adjusted higher criticism score from [2]
+
+        Args:
+        -----
+        'gamma' : lower fraction of P-values to consider
+
+        Returns:
+        -------
+        HC score, P-value attaining it
+
+        """
+
+        imin = np.argmax(self._pvals > np.log(self._N) / self._N)
+        imax = np.maximum(imin + 1,
+                        int(np.floor(gamma * self._N + 0.5)))
+        return self._calculateHC(imin, imax)
+    
+    def BJ(self, gamma=0.5) :
         """
         Exact Berk-Jones statistic
 
@@ -97,15 +119,13 @@ class HC(object) :
         p_th = spv[0]
             
         ii = np.arange(1, N + 1)
-        max_i = max(1, int(gamma * len(ii)))
-
-        if len(spv) >= 1 :
-            BJpv = beta.sf(spv, ii, N - ii + 1)[:max_i]
-            i_star = np.argmin(BJpv)
-            bj = BJpv[i_star]
-            p_th = spv[i_star]
+        max_i = int(N * gamma)
         
-        return -np.log(bj), p_th
+        if len(spv) >= 1 :
+            BJpv = beta.cdf(spv, ii, N - ii + 1)[:max_i]
+            bj = np.minimum(np.min(NJpv), np.min(1-NJpv))
+            
+        return bj
 
     def HCstar(self, gamma=0.2) :
         """sample-adjusted higher criticism score
@@ -134,6 +154,45 @@ class HC(object) :
                 }
 
 
+def two_sample_test(smp1, smp2, data_type = 'counts',
+                         alt='two-sided', **kwargs) :
+    """
+    Returns HC score and HC threshold in a two-sample test. 
+    =========================================================
+
+    Args:
+    ----
+
+    smp1, smp2    dataset representing samples from the identical or different
+                  populations.
+    data_type     either 'counts' of categorical variables or 'reals'
+    alt           how to compute P-values ('two-sided' or 'greater')
+    kwargs        additional arguments for the class HC and pvalue computation
+
+    Returns:
+    -------
+    (HC, HCT)     HC score, HC threshold P-value
+
+    """
+
+    stbl = kwargs.get('stbl', True)
+    randomize = kwargs.get('randomize', False)
+    gamma = kwargs.get('gamma', 0.35)
+
+    smp1 = np.array(smp1)
+    smp2 = np.array(smp2)
+
+    if data_type == 'counts' :
+        pvals = two_sample_pvals(smp1, smp2, randomize=randomize, alt=alt)
+    elif data_type == 'reals' :
+        z = (smp1 - smp2) / np.sqrt(2)
+        if alt == 'greater' :
+            pvals = norm.sf(z)
+        else :
+            pvals = norm.sf(np.abs(z))
+
+    return HC(pvals[~np.isnan(pvals)], stbl).HCstar(gamma)
+
 def hc_vals(pv, gamma=0.2, minPv='one_over_n', stbl=True):
     """
     Higher Criticism test (see
@@ -201,6 +260,37 @@ def hc_vals(pv, gamma=0.2, minPv='one_over_n', stbl=True):
 
     return hc_star, p_star
 
+def binom_test(x, n, p, alt='greater') :
+    """
+    Returns:
+    --------
+    Prob(Bin(n,p) >= x) ('greater')
+    or Prob(Bin(n,p) <= x) ('less')
+
+    Note: for small values of Prob there are differences
+    fron scipy.python.binom_test. It is unclear which one is 
+    more accurate.
+    """
+    n = n.astype(int)
+    if alt == 'greater' :
+        return binom.sf(x, n, p) + binom.pmf(x, n, p)
+    if alt == 'less' :
+        return binom.cdf(x, n, p)
+
+def exact_multinomial_test(x, p) : # slow
+    import met
+    assert(len(x) == len(p))
+    
+    n_max = 50
+    p = np.array(p) / np.sum(p)
+    n = sum(x)
+    if n > n_max :
+        return chisquare(x, p * n)[1]
+    all_multi_cases = [tup[0] for tup in met.onesided_exact_likelihood(x, [1,1,1])]
+    probs = multinomial.pmf(x=all_multi_cases, n=n, p=p)
+    pval = probs[probs <= probs[all_multi_cases.index(list(x))]].sum()
+    return pval
+
 
 def binom_test_two_sided_slow(x, n, p) :
     """
@@ -210,7 +300,6 @@ def binom_test_two_sided_slow(x, n, p) :
     """
     #slower
     def my_func(r) :
-        from scipy.stats import binom_test
         return binom_test(r[0],r[1],r[2])
 
     a = np.concatenate([np.expand_dims(x,1),
@@ -223,11 +312,12 @@ def binom_test_two_sided_slow(x, n, p) :
     return pv
 
 def poisson_test_random(x, lmd) :
+    """Prob( Pois(n,p) >= x ) + randomization """
     p_down = 1 - poisson.cdf(x, lmd)
     p_up = 1 - poisson.cdf(x, lmd) + poisson.pmf(x, lmd)
     U = np.random.rand(x.shape[0])
     prob = np.minimum(p_down + (p_up-p_down)*U, 1)
-    return prob * (n != 0) + U * (n == 0)
+    return prob * (x != 0) + U * (x == 0)
 
 def binom_test_two_sided(x, n, p) :
     """
@@ -276,37 +366,40 @@ def binom_test_two_sided_random(x, n, p) :
     prob = np.minimum(p_down + (p_up-p_down)*U, 1)
     return prob * (n != 0) + U * (n == 0)
 
-def two_sample_test(X, Y, gamma=0.25,
-                stbl=True, randomize=False):
-    """
-    Two-sample HC test using binomial P-values. See 
-    [1] Alon Kipnis, ``Higher Criticism for Discriminating Word-Frequency
-    Tables and Testing Authorship'', 2019
+# def two_sample_test(X, Y, gamma=0.25,
+#                 stbl=True, randomize=False,
+#                 alt='two-sided') :
+#     """
+#     Two-sample HC test using binomial P-values. See 
+#     [1] Alon Kipnis, ``Higher Criticism for Discriminating Word-Frequency
+#     Tables and Testing Authorship'', 2019
     
-    This function combines two_sample_pvals and hc_vals.
+#     This function combines two_sample_pvals and hc_vals.
 
-    Args:
-    -----
-    X, Y : list of integers of equal length -- represnts counts 
-            from two samples.
-    gamma : number in (0,1) -- parameter of HC statistics
-    stbl : Boolean -- standardize P-values by i/N or p_{(i)}/N
-    randomize : Boolean -- randomized P-valus of not
+#     Args:
+#     -----
+#     X, Y : list of integers of equal length -- represnting counts 
+#             from two samples.
+#     gamma : number in (0,1) -- parameter of HC statistics
+#     stbl : Boolean -- standardize P-values by i/N or p_{(i)}/N
+#     randomize : Boolean -- randomized P-valus of not
+#     alt  :   how to compute P-values ('two-sided' or 'greater')
 
-    Returns:
-    --------
-    HC : HC score under Binmial P-values
-    p_thresh : HC threshold
-    """
+#     Returns:
+#     --------
+#     HC : HC score under Binmial P-values
+#     p_thresh : HC threshold
+#     """
     
-    pvals = two_sample_pvals(X, Y, randomize=randomize)
-    hc_star, p_thresh = hc_vals(pvals, gamma=gamma, stbl=stbl)
+#     pvals = two_sample_pvals(X, Y, randomize=randomize, alt=alt)
+#     hc_star, p_thresh = HC(pvals[~np.isnan(pvals)], stbl).HCstar(gamma)
+#     #hc_star, p_thresh = hc_vals(pvals, gamma=gamma, stbl=stbl)
     
-    return hc_star, p_thresh
+#     return hc_star, p_thresh
 
 def binom_var_test_df(c1, c2, sym=False, max_m=-1) :
     """ Binmial variance test along stripes. 
-        This version returns all quantities in the calculation
+        This version returns all sub-calculations
     Args:
     ----
     c1, c2 : list of integers represents count data from two sample
@@ -316,16 +409,14 @@ def binom_var_test_df(c1, c2, sym=False, max_m=-1) :
 
     df_smp = pd.DataFrame({'n1' : c1, 'n2' : c2})
     df_smp.loc[:,'N'] = df_smp.agg('sum', axis = 'columns')
-
+    
     if max_m > 0 :
         df_smp = df_smp[df_smp.n1 + df_smp.n2 <= max_m]
-
+        
     df_hist = df_smp.groupby(['n1', 'n2']).count().reset_index()
-    # truncate where normal approximation is no longer valid
-    df_hist = df_hist[df_hist.N > np.minimum(df_hist.n1 + df_hist.n2, 9)]
-
     df_hist.loc[:,'m'] = df_hist.n1 + df_hist.n2
-
+    df_hist = df_hist[df_hist.m > 0]
+    
     df_hist.loc[:,'N1'] = df_hist.n1 * df_hist.N
     df_hist.loc[:,'N2'] = df_hist.n2 * df_hist.N
 
@@ -334,9 +425,10 @@ def binom_var_test_df(c1, c2, sym=False, max_m=-1) :
 
     df_hist = df_hist.join(df_hist.filter(['m', 'N1', 'N2', 'N']).groupby('m').agg('sum'),
                            on = 'm', rsuffix='_m')
+    if max_m == -1 :
+        df_hist = df_hist[df_hist.N_m > np.maximum(df_hist.n1, df_hist.n2)]
 
-    df_hist.loc[:,'p'] = (df_hist['NN1'] - df_hist['N1_m'])\
-            / (df_hist['NN1'] + df_hist['NN2'] - df_hist['N1_m'] - df_hist['N2_m'])
+    df_hist.loc[:,'p'] = df_hist['NN1'] / (df_hist['NN1'] + df_hist['NN2'])
 
     df_hist.loc[:,'s'] = (df_hist.n1 - df_hist.m * df_hist.p) ** 2 * df_hist.N
     df_hist.loc[:,'Es'] = df_hist.N_m * df_hist.m * df_hist.p * (1 - df_hist.p)
@@ -365,7 +457,8 @@ def binom_var_test(c1, c2, sym=False, max_m=-1) :
     df_hist = binom_var_test_df(c1, c2, sym=sym, max_m=max_m)
     return df_hist.groupby('m').pval.mean()
 
-def two_sample_pvals(c1, c2, randomize=False, sym=False):
+def two_sample_pvals(c1, c2, randomize=False,
+     sym=False, alt='two-sided', ret_p=False):
 
     """ feature by feature exact binomial test
     Args:
@@ -374,7 +467,9 @@ def two_sample_pvals(c1, c2, randomize=False, sym=False):
     randomize : flag indicate wether to use randomized P-values
     sym : flag indicates wether the size of both sample is assumed
           identical, hence p=1/2
+    alt :  how to compute P-values. 
     """
+
     T1 = c1.sum()
     T2 = c2.sum()
 
@@ -382,20 +477,23 @@ def two_sample_pvals(c1, c2, randomize=False, sym=False):
     if den.sum() == 0 :
         return c1 * np.nan
 
-    p = ((T1 - c1) / den)*(1-sym) + sym * 1./2
+    p = ((T1 - c1) / den)*(not sym) + sym * 1./2
 
-    if randomize :
+    if alt == 'greater' or alt == 'less' :
+        pvals = binom_test(c1, c1 + c2, p, alt=alt)
+    elif randomize :
         pvals = binom_test_two_sided_random(c1, c1 + c2, p)
     else :
         pvals = binom_test_two_sided(c1, c1 + c2, p)
-        #pvals = binom_test_two_sided_slow(c1, c1 + c2, p)
 
+    if ret_p :
+        return pvals, p
     return pvals
 
 def two_sample_test_df(X, Y, gamma=0.25, min_cnt=0,
-                stbl=True, randomize=False):
+                stbl=True, randomize=False, 
+                alt='two-sided'):
     """
-
     Same as two_sample_test but returns all information for computing
     HC score of the two samples as a pandas data DataFrame. 
     Requires pandas.
@@ -434,15 +532,12 @@ def two_sample_test_df(X, Y, gamma=0.25, min_cnt=0,
     T1 = counts['n1'].sum()
     T2 = counts['n2'].sum()
     
-    counts['p'] = (T1 - counts.n1) / (T1 + T2 - counts.n1 - counts.n2)
-
     counts['T1'] = T1
     counts['T2'] = T2
 
-    counts['pval'] = two_sample_pvals(
-        counts['n1'],
-        counts['n2'],
-        randomize=randomize
+    counts['pval'], counts['p'] = two_sample_pvals(
+        counts['n1'], counts['n2'],
+        randomize=randomize, alt=alt, ret_p=True
         )
 
     counts['sign'] = np.sign(counts.n1 - (counts.n1 + counts.n2) * counts.p)
